"""Node parsers."""

from file.html import HTMLNodeParser
from file.json import JSONNodeParser
from file.markdown import MarkdownNodeParser
from file.simple_file import SimpleFileNodeParser
from interface import (
    MetadataAwareTextSplitter,
    NodeParser,
    TextSplitter,
)
from .relational.hierarchical import (
    HierarchicalNodeParser,
    get_leaf_nodes,
    get_root_nodes,
    get_child_nodes,
    get_deeper_nodes,
)
from .relational.markdown_element import (
    MarkdownElementNodeParser,
)
from .relational.unstructured_element import (
    UnstructuredElementNodeParser,
)
from .relational.llama_parse_json_element import (
    LlamaParseJsonNodeParser,
)
from .text.code import CodeSplitter
from .text.langchain import LangchainNodeParser
from .text.semantic_splitter import (
    SemanticSplitterNodeParser,
)
from .text.semantic_double_merging_splitter import (
    SemanticDoubleMergingSplitterNodeParser,
    LanguageConfig,
)
from .text.sentence import SentenceSplitter
from .text.sentence_window import (
    SentenceWindowNodeParser,
)
from .text.token import TokenTextSplitter

# deprecated, for backwards compatibility
SimpleNodeParser = SentenceSplitter

__all__ = [
    "TokenTextSplitter",
    "SentenceSplitter",
    "CodeSplitter",
    "SimpleFileNodeParser",
    "HTMLNodeParser",
    "MarkdownNodeParser",
    "JSONNodeParser",
    "SentenceWindowNodeParser",
    "SemanticSplitterNodeParser",
    "SemanticDoubleMergingSplitterNodeParser",
    "LanguageConfig",
    "NodeParser",
    "HierarchicalNodeParser",
    "TextSplitter",
    "MarkdownElementNodeParser",
    "MetadataAwareTextSplitter",
    "LangchainNodeParser",
    "UnstructuredElementNodeParser",
    "get_leaf_nodes",
    "get_root_nodes",
    "get_child_nodes",
    "get_deeper_nodes",
    "LlamaParseJsonNodeParser",
    # deprecated, for backwards compatibility
    "SimpleNodeParser",
]
